<?php

namespace Drupal\layout_builder_base\Plugin\Layout;

use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\layout_builder\Plugin\Layout\MultiWidthLayoutBase;

/**
 * Configurable layout plugin class.
 *
 * @internal
 *   Plugin classes are internal.
 */
abstract class BaseLayoutBase extends MultiWidthLayoutBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = [
      'background' => $this->getDefaultBackground(),
      'background_image' => $this->getDefaultImageBackground(),
      'equal_top_bottom_margin' => $this->getDefaultEqualTopBottomMargin(),
      'equal_left_right_margin' => $this->getDefaultEqualLeftRightMargin(),
      'top_margin' => $this->getDefaultTopMargin(),
      'right_margin' => $this->getDefaultRightMargin(),
      'bottom_margin' => $this->getDefaultBottomMargin(),
      'left_margin' => $this->getDefaultLeftMargin(),
      'equal_top_bottom_padding' => $this->getDefaultEqualTopBottomPadding(),
      'equal_left_right_padding' => $this->getDefaultEqualLeftRightPadding(),
      'top_padding' => $this->getDefaultTopPadding(),
      'right_padding' => $this->getDefaultRightPadding(),
      'bottom_padding' => $this->getDefaultBottomPadding(),
      'left_padding' => $this->getDefaultLeftPadding(),
      'container' => $this->getDefaultContainer(),
      'color' => $this->getDefaultColor(),
    ];

    return array_merge(parent::defaultConfiguration(), $config);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    if (!empty($this->getWidthOptions())) {
      $form = parent::buildConfigurationForm($form, $form_state);
    }

    if ($this->enableImageBackground()) {
      $form['background_image'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Background image'),
        '#default_value' => (!empty($this->configuration['background_image'])) ? [$this->configuration['background_image']] : '',
        '#description' => $this->t('Choose a background image for this layout. This feature is using inline CSS.'),
        '#upload_validators' => [
          'file_validate_is_image' => [],
          'file_validate_extensions' => ['gif png jpg jpeg'],
        ],
        '#upload_location' => 'public://layout_background/',
      ];
    }

    if (!empty($this->getBackgroundOptions())) {
      $form['background'] = [
        '#type' => 'select',
        '#title' => $this->t('Background'),
        '#default_value' => $this->configuration['background'],
        '#options' => $this->getBackgroundOptions(),
        '#description' => $this->t('Choose the background for this layout.'),
      ];
    }

    if (!empty($this->getEqualTopBottomMarginsOptions())) {
      $form['equal_top_bottom_margin'] = [
        '#type' => 'select',
        '#title' => $this->t('Top Bottom Margins'),
        '#default_value' => $this->configuration['equal_top_bottom_margin'],
        '#options' => $this->getEqualTopBottomMarginsOptions(),
        '#description' => $this->t('Choose the top/bottom margins for this layout.'),
      ];
    }

    if (!empty($this->getEqualLeftRightMarginsOptions())) {
      $form['equal_left_right_margin'] = [
        '#type' => 'select',
        '#title' => $this->t('Left Right Margins'),
        '#default_value' => $this->configuration['equal_left_right_margin'],
        '#options' => $this->getEqualLeftRightMarginsOptions(),
        '#description' => $this->t('Choose the left/right margins for this layout.'),
      ];
    }

    if (!empty($this->getEqualTopBottomPaddingsOptions())) {
      $form['equal_top_bottom_padding'] = [
        '#type' => 'select',
        '#title' => $this->t('Top Bottom Paddings'),
        '#default_value' => $this->configuration['equal_top_bottom_padding'],
        '#options' => $this->getEqualTopBottomPaddingsOptions(),
        '#description' => $this->t('Choose the top/bottom paddings for this layout.'),
      ];
    }

    if (!empty($this->getEqualLeftRightPaddingsOptions())) {
      $form['equal_left_right_padding'] = [
        '#type' => 'select',
        '#title' => $this->t('Left Right Paddings'),
        '#default_value' => $this->configuration['equal_left_right_padding'],
        '#options' => $this->getEqualLeftRightPaddingsOptions(),
        '#description' => $this->t('Choose the left/right paddings for this layout.'),
      ];
    }

    if (!empty($this->getContainerOptions())) {
      $form['container'] = [
        '#type' => 'select',
        '#title' => $this->t('Container'),
        '#default_value' => $this->configuration['container'],
        '#options' => $this->getContainerOptions(),
        '#description' => $this->t('Choose the size of the container for this layout.'),
      ];
    }

    if (!empty($this->getColorsOptions())) {
      $form['color'] = [
        '#type' => 'select',
        '#title' => $this->t('Default color'),
        '#default_value' => $this->configuration['color'],
        '#options' => $this->getColorsOptions(),
        '#description' => $this->t('Choose the default color for the text of this layout.'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $background_file = $form_state->getValue('background_image', 0);
    if (isset($background_file[0]) && !empty($background_file[0])) {
      $file = File::load($background_file[0]);
      $file->setPermanent();
      $file->save();
    }

    $this->configuration['background_image'] = !empty($file) ? $file->id() : '';
    $this->configuration['background'] = $form_state->getValue('background');
    $this->configuration['equal_top_bottom_margin'] = $form_state->getValue('equal_top_bottom_margin');
    $this->configuration['equal_left_right_margin'] = $form_state->getValue('equal_left_right_margin');
    $this->configuration['equal_top_bottom_padding'] = $form_state->getValue('equal_top_bottom_padding');
    $this->configuration['equal_left_right_padding'] = $form_state->getValue('equal_left_right_padding');
    $this->configuration['container'] = $form_state->getValue('container');
    $this->configuration['color'] = $form_state->getValue('color');
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);
    $classes = [
      $this->configuration['background'],
      $this->configuration['equal_top_bottom_margin'],
      $this->configuration['equal_left_right_margin'],
      $this->configuration['equal_top_bottom_padding'],
      $this->configuration['equal_left_right_padding'],
      $this->configuration['container'],
      $this->configuration['color'],
    ];
    $build_classes = array_merge($build['#attributes']['class'], $classes);
    $build['#attributes']['class'] = $build_classes;

    if (!empty($this->configuration['background_image'])) {
      $image = File::load($this->configuration['background_image']);
      if (!empty($image)) {
        $url = file_create_url($image->uri->value);
        $build['#attributes']['style'] = 'background-image: url("' . $url . '")';
      }
    }

    return $build;
  }

  /**
   * Default behavior to get the default config option.
   *
   * @return string
   *   Return the first option.
   */
  protected function getDefaultConfigOption($options) {
    return !empty($options) ? array_shift($options) : '';
  }

  /**
   * Get the default background option.
   *
   * @return string
   *   Return the default background option.
   */
  protected function getDefaultBackground() {
    $options = $this->getBackgroundOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default background option.
   *
   * @return string
   *   Return the default background option.
   */
  protected function getDefaultImageBackground() {
    return '';
  }

  /**
   * Get the default equal top bottom margin option.
   *
   * @return string
   *   Return the default equal top bottom margin option.
   */
  protected function getDefaultEqualTopBottomMargin() {
    $options = $this->getEqualTopBottomMarginsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default equal left right margin option.
   *
   * @return string
   *   Return the default equal left right margin option.
   */
  protected function getDefaultEqualLeftRightMargin() {
    $options = $this->getEqualLeftRightMarginsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default top margin option.
   *
   * @return string
   *   Return the default top margin option.
   */
  protected function getDefaultTopMargin() {
    $options = $this->getTopMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default right margin option.
   *
   * @return string
   *   Return the default right margin option.
   */
  protected function getDefaultRightMargin() {
    $options = $this->getRightMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default bottom margin option.
   *
   * @return string
   *   Return the default bottom margin option.
   */
  protected function getDefaultBottomMargin() {
    $options = $this->getBottomMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default left margin option.
   *
   * @return string
   *   Return the default left margin option.
   */
  protected function getDefaultLeftMargin() {
    $options = $this->getLeftMarginOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default equal top bottom padding option.
   *
   * @return string
   *   Return the default equal top bottom padding option.
   */
  protected function getDefaultEqualTopBottomPadding() {
    $options = $this->getEqualTopBottomPaddingsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default equal left right padding option.
   *
   * @return string
   *   Return the default equal left right padding option.
   */
  protected function getDefaultEqualLeftRightPadding() {
    $options = $this->getEqualLeftRightPaddingsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default top padding option.
   *
   * @return string
   *   Return the default top padding option.
   */
  protected function getDefaultTopPadding() {
    $options = $this->getTopPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default right padding option.
   *
   * @return string
   *   Return the default right padding option.
   */
  protected function getDefaultRightPadding() {
    $options = $this->getRightPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default bottom padding option.
   *
   * @return string
   *   Return the default bottom padding option.
   */
  protected function getDefaultBottomPadding() {
    $options = $this->getBottomPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default left padding option.
   *
   * @return string
   *   Return the default left padding option.
   */
  protected function getDefaultLeftPadding() {
    $options = $this->getLeftPaddingOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default container option.
   *
   * @return string
   *   Return the default container option.
   */
  protected function getDefaultContainer() {
    $options = $this->getContainerOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Get the default color option.
   *
   * @return string
   *   Return the default color option.
   */
  protected function getDefaultColor() {
    $options = $this->getColorsOptions();
    return $this->getDefaultConfigOption($options);
  }

  /**
   * Function to enable the background image feature.
   *
   * Override this function and return TRUE in order to enable the image
   * background feature.
   *
   * @return bool
   *   Boolean to know if the image background feature is enabled or not.
   */
  protected function enableImageBackground() {
    return FALSE;
  }

  /**
   * Gets the background options for the configuration form.
   *
   * @return string[]
   *   The background options array where the keys are strings that will
   *   be added to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getBackgroundOptions();

  /**
   * Gets the top and bottom equal margin options for the configuration form.
   *
   * @return string[]
   *   The top and bottom equal margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualTopBottomMarginsOptions();

  /**
   * Gets the left and right equal margin options for the configuration form.
   *
   * @return string[]
   *   The left and right equal margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualLeftRightMarginsOptions();

  /**
   * Gets the top margin options for the configuration form.
   *
   * @return string[]
   *   The top margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getTopMarginOptions();

  /**
   * Gets the right margin options for the configuration form.
   *
   * @return string[]
   *   The right margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getRightMarginOptions();

  /**
   * Gets the bottom margin options for the configuration form.
   *
   * @return string[]
   *   The bottom margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getBottomMarginOptions();

  /**
   * Gets the left margin options for the configuration form.
   *
   * @return string[]
   *   The left margin options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getLeftMarginOptions();

  /**
   * Gets the top and bottom equal padding options for the configuration form.
   *
   * @return string[]
   *   The top and bottom equal padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualTopBottomPaddingsOptions();

  /**
   * Gets the left and right equal padding options for the configuration form.
   *
   * @return string[]
   *   The left and right equal padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getEqualLeftRightPaddingsOptions();

  /**
   * Gets the top padding options for the configuration form.
   *
   * @return string[]
   *   The top padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getTopPaddingOptions();

  /**
   * Gets the right padding options for the configuration form.
   *
   * @return string[]
   *   The right padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getRightPaddingOptions();

  /**
   * Gets the bottom padding options for the configuration form.
   *
   * @return string[]
   *   The bottom padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getBottomPaddingOptions();

  /**
   * Gets the left padding options for the configuration form.
   *
   * @return string[]
   *   The left padding options array where the keys are strings
   *   that will be added to the CSS classes and the values are the human
   *   readable labels.
   */
  abstract protected function getLeftPaddingOptions();

  /**
   * Gets the container options for the configuration form.
   *
   * @return string[]
   *   The container options array where the keys are strings that will be added
   *   to the CSS classes and the values are the human readable labels.
   */
  abstract protected function getContainerOptions();

  /**
   * Gets the colors options for the configuration form.
   *
   * @return string[]
   *   The colors options array where the keys are strings that will be added to
   *   the CSS classes and the values are the human readable labels.
   */
  abstract protected function getColorsOptions();

}
